/*
 * Name: File drop upload
 * Developer: Andros Haggins, Shreyas Kumar, and Michael Spears
 * Date: 9/6/2022
 
 * File drop upload is a widget that
 */
define(function () {

    var install = function (holderElement, context) {
        var FilePond = context.loadedDependencies[0]; // FilePond library
        var $ = context.loadedDependencies[1]; // JQuery
        var FilePondPluginImageExifOrientation = context.loadedDependencies[2]; // exif reader plugin
        var FilePondPluginImagePreview = context.loadedDependencies[3]; // image preview plugin
        var FilePondPluginFileValidateSize = context.loadedDependencies[4]; // file size validation plugin

      

        var parameters = {
            allowMultiple: parseInt(context.getParameter("allowMultiple"), 10) === 1,
            runSave: parseInt(context.getParameter("runSave"), 10) === 1,
            forceReload: parseInt(context.getParameter("forceReload"), 10) === 1,
            allowImagePreview: parseInt(context.getParameter("allowImagePreview"), 10) === 1,
            allowFileSizeValidation: parseInt(context.getParameter("allowFileSizeValidation"), 10) === 1,
            maxFileSize: context.getParameter("maxFileSize"),
            
        };

       
        // Add two plugins to do an image preview and size validation 
        FilePond.registerPlugin(
            FilePondPluginImagePreview,
            FilePondPluginFileValidateSize,
            FilePondPluginImageExifOrientation
        );

        const filePond = FilePond.create(
            // Apply to the <input type=file>
            holderElement.childNodes[0],
            // These are the options, you can add more parameters here as needed.
            {
                allowMultiple: parameters.allowMultiple,
                allowImagePreview: parameters.allowImagePreview,
                allowFileSizeValidation: parameters.allowFileSizeValidation,
                maxFileSize: parameters.maxFileSize,
            }
        );
       
       
        

        // Function passed into the processor to know if the file pond is uploading anything
        const isComplete = (file) => {
            // Look and see if all the files are completed. Why this isn't a method on FilePond is surprising
            var files = filePond.getFiles();
            var incompleteFiles = files.filter(file2 => file2.status !== /*complete*/5);

            // All files done and we just completed this one
            // Again, even after telling above we're complete, the last processed file is still "processing"
            return incompleteFiles.length === 1 && incompleteFiles[0].file === file;
        };

        // Take over the file upload process
        // Passing in the Vinyl internal FileViewModel and RowViewModel to run the change & save
        filePond.server = {
            process: getProcessHandler(context.viewModel, context.getRow().viewModel, isComplete, parameters)
        };
    };

    var uninstall = function (holderElement, context) {
        var FilePond = context.loadedDependencies[0]; // FilePond library

        // Run destroy on the <input>
        FilePond.destroy(holderElement.childNodes[0]);
    };

    // Get a process handler function
    var getProcessHandler = function (fileViewModel, rowViewModel, isComplete, parameters) {
        // A promise used to ensure the file Vinyl saving happens serially
        var _currentSavePromise = undefined;
       
        // this returns a function that will be called when a file has been added to the drop zone
        return (fieldName, file, metadata, load, error, progress, abort) => {
            // 1/8 complete once we've got the drop request
            progress(/*computed*/true, 1, 8);

            const reader = new FileReader();
            
            // A handler that runs when the file has been converted to base64
            // Leaving this here because it accesses so many parameters
            const fileConvertedHandler = () => {

                progress(/*computed*/true, 1, 4); // 25%

                // The result has been converted to a data url (base64 encoded)
                const base64String = reader.result.replace('data:', '').replace(/^.+,/, '');

                // Pass the file (to get the filename) and the base64 string to the Vinyl FileViewModel
                fileViewModel.queueFileUpload(file, base64String);
                // if we're not running save, we're using the FilePond to only replace a edit state file dropper into the cell
                if (!parameters.runSave) {
                    load(file.name);
                    return $.when();
                }

                progress(/*computed*/true, 1, 2); // 50%

                // Run the save, do not permit reloading of source data
                return rowViewModel.beginSave(/*disallow bitwise*/1)
                    // Internal promise to detect when the event actually completes (including warnings)
                    .then(_ => _.onEventComplete())
                    // Fail handler, logs and passes back to the FilePond callback
                    .catch((result) => {
                        console.error("Error saving file", result);
                        error("Error saving file");
                    })
                    .then(_ => {
                        // Pass the filename back to FilePond so it knows this file is complete (sets to 100%)
                        load(file.name);

                    if (parameters.forceReload && isComplete(file)) {
                        // all completed
                        // check this for the daikon ticket
                        rowViewModel.source.constraint.requestReload();
                    }
            });
        };

            reader.onloadend = () => {
                // delay the loadPage call until the active one is completed.
                _currentSavePromise = Promise.resolve(_currentSavePromise)
                    .then(fileConvertedHandler);
            };

            // Pass reader errors back to the FilePond
            reader.onerror = error;

            // Take the Blob FilePond gave us and convert to a base64 string
            reader.readAsDataURL(file);

            // Should expose an abort method so the request can be cancelled
            return {
                abort: () => {
                    // Handle abort, not implemented

                    // Let FilePond know the request has been cancelled
                    abort();
                }
            };
        };
    };

    return {
        callbacks: {
            events: {
                // adding and removing
                install: install,
                uninstall: uninstall
            }
        }
    };
});